class SignalGenerator {
  constructor() {
    this.initializeCanvases();
    this.setupEventListeners();
    this.drawInitialSignals();
  }

  initializeCanvases() {
    this.timeCanvas = document.getElementById('time-domain');
    this.freqCanvas = document.getElementById('frequency-domain');
    this.timeCtx = this.timeCanvas.getContext('2d');
    this.freqCtx = this.freqCanvas.getContext('2d');
    
    this.resizeCanvases();
    window.addEventListener('resize', () => this.resizeCanvases());
  }

  resizeCanvases() {
    const setCanvasSize = (canvas) => {
      canvas.width = canvas.offsetWidth;
      canvas.height = canvas.offsetHeight;
    };
    setCanvasSize(this.timeCanvas);
    setCanvasSize(this.freqCanvas);
  }

  setupEventListeners() {
    const controls = ['f1-type', 'f1-frequency', 'f1-amplitude', 
                     'f2-type', 'f2-frequency', 'f2-amplitude'];
    
    controls.forEach(id => {
      const element = document.getElementById(id);
      element.addEventListener('input', () => this.updateSignals());
      
      if (id.includes('frequency') || id.includes('amplitude')) {
        element.addEventListener('input', () => {
          const valueDisplay = document.getElementById(`${id}-value`);
          const value = element.value;
          valueDisplay.textContent = id.includes('frequency') ? 
            `${value} Hz` : value;
        });
      }
    });

    document.getElementById('btn-add').addEventListener('click', () => {
      this.operation = 'add';
      this.updateSignals();
    });

    document.getElementById('btn-multiply').addEventListener('click', () => {
      this.operation = 'multiply';
      this.updateSignals();
    });

    document.getElementById('btn-reset').addEventListener('click', () => {
      this.operation = null;
      this.updateSignals();
    });
  }

  generateSignal(type, frequency, amplitude, t) {
    switch(type) {
      case 'sine':
        return amplitude * Math.sin(2 * Math.PI * frequency * t);
      case 'square':
        return amplitude * Math.sign(Math.sin(2 * Math.PI * frequency * t));
      case 'triangle':
        return amplitude * (Math.asin(Math.sin(2 * Math.PI * frequency * t)) * 2 / Math.PI);
      case 'sawtooth':
        return amplitude * (2 * (frequency * t - Math.floor(0.5 + frequency * t)));
      default:
        return 0;
    }
  }

  drawSignal(ctx, signal, color) {
    ctx.beginPath();
    ctx.strokeStyle = color;
    ctx.lineWidth = 2;

    for (let x = 0; x < ctx.canvas.width; x++) {
      const t = x / ctx.canvas.width;
      const y = (signal(t) + 1) * ctx.canvas.height / 2;
      x === 0 ? ctx.moveTo(x, y) : ctx.lineTo(x, y);
    }

    ctx.stroke();
  }

  calculateFFT(signal) {
    const N = 512;
    const data = new Array(N);
    
    for (let i = 0; i < N; i++) {
      data[i] = signal(i / N);
    }

    // Simple DFT implementation (for demonstration)
    const spectrum = new Array(N/2);
    for (let k = 0; k < N/2; k++) {
      let real = 0, imag = 0;
      for (let n = 0; n < N; n++) {
        const angle = 2 * Math.PI * k * n / N;
        real += data[n] * Math.cos(angle);
        imag -= data[n] * Math.sin(angle);
      }
      spectrum[k] = Math.sqrt(real*real + imag*imag) / N;
    }
    
    return spectrum;
  }

  drawSpectrum(ctx, spectrum) {
    ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);
    
    const barWidth = ctx.canvas.width / spectrum.length;
    const maxMagnitude = Math.max(...spectrum);
    
    ctx.fillStyle = '#3498db';
    spectrum.forEach((magnitude, i) => {
      const height = (magnitude / maxMagnitude) * ctx.canvas.height;
      ctx.fillRect(
        i * barWidth,
        ctx.canvas.height - height,
        barWidth - 1,
        height
      );
    });
  }

  updateSignals() {
    const f1Type = document.getElementById('f1-type').value;
    const f1Freq = parseFloat(document.getElementById('f1-frequency').value);
    const f1Amp = parseFloat(document.getElementById('f1-amplitude').value);
    
    const f2Type = document.getElementById('f2-type').value;
    const f2Freq = parseFloat(document.getElementById('f2-frequency').value);
    const f2Amp = parseFloat(document.getElementById('f2-amplitude').value);

    const signal1 = (t) => this.generateSignal(f1Type, f1Freq, f1Amp, t);
    const signal2 = (t) => this.generateSignal(f2Type, f2Freq, f2Amp, t);

    let combinedSignal;
    if (this.operation === 'add') {
      combinedSignal = (t) => signal1(t) + signal2(t);
    } else if (this.operation === 'multiply') {
      combinedSignal = (t) => signal1(t) * signal2(t);
    }

    // Clear canvases
    this.timeCtx.clearRect(0, 0, this.timeCanvas.width, this.timeCanvas.height);
    
    // Draw signals
    this.drawSignal(this.timeCtx, signal1, '#3498db');
    this.drawSignal(this.timeCtx, signal2, '#e74c3c');
    if (combinedSignal) {
      this.drawSignal(this.timeCtx, combinedSignal, '#2ecc71');
    }

    // Draw spectrum
    const spectrum = this.calculateFFT(combinedSignal || signal1);
    this.drawSpectrum(this.freqCtx, spectrum);
  }

  drawInitialSignals() {
    this.updateSignals();
  }
}

document.addEventListener('DOMContentLoaded', () => {
  new SignalGenerator();
});
